mode 1,8 : cls
'======================================
size = 250
d = 800     ' distance to view plane
rho = 1000  ' distance to object (center)
theta = 0   ' angle around z-axis to viewer
phi = 0     ' angle down from 'north' to viewer
roll = 0    ' roll angle around line of sight
cenx = MM.HRES/2
ceny = MM.VRES/2
X = 0 : Y = 1 : Z = 2  ' used to access array index
toggle = 0

dim soldvert(50,2)   ' vertices - 3D
dim projvert(50,1)   ' vertices projected onto viewing plane - 2D
dim vertvis(50)      ' visibility flags for labeling
dim faces(50,2)      ' faces

LoadShape(2)

do
  cls

  '========================================
  ' draw fixed object, rotating viewer
  ' theta, phi and roll are the angles of the viewer from the origin
  ' rho is the distance of the viewer from the origin
  ' d is the distance of the view plane from the viewer
  print @(0,0) "(j/k) Roll: ", roll
  print "(w/x) Pitch:", phi
  print "(a/d) Yaw:  ", theta
  print "(i/m) Rho:  ", rho
  print "(u/n) View: ", d

  s1 = sin(rad(theta)) : c1 = cos(rad(theta))
  s2 = sin(rad(phi))   : c2 = cos(rad(phi))
  s3 = sin(rad(roll))  : c3 = cos(rad(roll))

  math set 0, vertvis()   ' mark all not visible to start
  
  project6    ' project all vertices
  for j = 0 to nfaces  ' draw faces
  
    ' pull out projected x,y of three corners of tiangular face
    x0 = projvert(faces(j,0),X) : y0 = projvert(faces(j,0),Y)
    x1 = projvert(faces(j,1),X) : y1 = projvert(faces(j,1),Y)
    x2 = projvert(faces(j,2),X) : y2 = projvert(faces(j,2),Y)
    
    ' test for visibility
    if (x0-x1)*(y0-y2)<(x0-x2)*(y0-y1) then
      triangle x0,y0,x1,y1,x2,y2,&hFFFFFF,0   ' draw face
      
      ' label faces and vertices?
      if toggle then
        text (x0+x1+x2)/3,(y0+y1+y2)/3,str$(j),"CM",,,rgb(green)  ' label face
        vertvis(faces(j,0)) = 1   ' mark these
        vertvis(faces(j,1)) = 1   '   3 vertices
        vertvis(faces(j,2)) = 1   '   as visible - for labeling
      endif
    endif
  next j
  
  ' label visible vertices
  if toggle then
    for j = 0 to nverts
      if vertvis(j) then
        LabelVert(projvert(j,X)+5,projvert(j,Y)+5,j,rgb(red))
      endif
    next j
  endif

  '========================================
  do : k$ = inkey$ : loop until k$ <> ""
  select case k$
    case "w"    ' pitch up
      phi = phi + 2 : if phi > 360 then phi = phi - 360
    case "x"    ' pitch down
      phi = phi - 2 : if phi <   0 then phi = phi + 360
    case "a"    ' yaw left
      theta = theta + 2 : if theta > 360 then theta = theta - 360
    case "d"    ' yaw right
      theta = theta - 2 : if theta <   0 then theta = theta + 360
    case "j"    ' roll left
      roll = roll + 2 : if roll > 360 then roll = roll - 360
    case "k"    ' roll right
      roll = roll - 2 : if roll <   0 then roll = roll + 360
    case "i"    ' zoom in
      rho = rho - 10 : if rho <  300 then rho =  300  ' don't get too close! Math fail.
    case "m"    ' zoom out
      rho = rho + 10 : if rho > 3000 then rho = 3000
    case "u"    ' view plane in
      d = d - 10 : if d <  300 then d =  300
    case "n"    ' view plane out
      d = d + 10 : if d > 3000 then d = 3000
    case "y"    ' both zoom and view plane in
      rho = rho - 10 : if rho <  300 then rho =  300  ' don't get too close! Math fail.
      d = d - 10 : if d <  300 then d =  300
    case "b"    ' both zoom and view plane out
      d = d + 10 : if d > 3000 then d = 3000
      rho = rho + 10 : if rho > 3000 then rho = 3000
    case "t"    ' toggle labels on/off
      toggle = toggle = 0
    case "1" to "3"
      LoadShape(val(k$))
  end select
loop

end

'========================================
sub project6
  for i = 0 to nverts
    xe = - soldvert(i,X) * s1 + soldvert(i,Y) * c1
    ye = - soldvert(i,X) * c1 * c2 - soldvert(i,Y) * s1 * c2 + soldvert(i,Z) * s2
    ze = - soldvert(i,X) * s2 * c1 - soldvert(i,Y) * s2 * s1 - soldvert(i,Z) * c2 + rho

    tx = d * xe / ze
    ty = d * ye / ze
    
    projvert(i,X) = (tx * c3 - ty * s3) + cenx
    projvert(i,Y) = ceny - (tx * s3 + ty * c3)
  next i
end sub

'========================================
sub LabelVert(x,y,n,c)
  local x2,y2
  
  x2 = x : y2 = y
  if x2 > MM.HRES/2 then
    x2 = x2 + 10
  else
    x2 = x2 - 10
  endif
  if y2 > MM.VRES/2 then
    y2 = y2 + 10
  else
    y2 = y2 - 10
  endif
  
  text x2,y2,str$(n),"CM",,,c  ' label vertex

end sub

'========================================
sub LoadShape(sn)
  local j
  
  select case sn
    case 1
      restore tetrashape
    case 3
      restore icosashape
    case else
      restore octashape
  end select
  
  ' read in vertices
  read nverts : nverts = nverts - 1
  for j = 0 to nverts
    read soldvert(j,X) : read soldvert(j,Y) : read soldvert(j,Z)
  next j
  math scale soldvert(), size, soldvert()

  ' read in faces
  read nfaces : nfaces = nfaces - 1
  for j = 0 to nfaces
    read faces(j,0) : read faces(j,1) : read faces(j,2)
  next j
  
end sub

'========================================
tetrashape:
' vertices (first value is count)
data  4, 0.576,0.576,0.576, -0.576,-0.576,0.576, -0.576,0.576,-0.576, 0.576,-0.576,-0.576
' faces (first value is count)
data  4, 0,2,1,  0,1,3,  0,3,2,  1,2,3

octashape:
data 6
' vertices    0      1       2       3      4      5
data        0,1,0, 1,0,0, -1,0,0, 0,-1,0, 0,0,1, 0,0,-1
' faces
data 8
data  4,2,3, 4,3,1, 4,1,0, 4,0,2
data  5,3,2, 5,1,3, 5,0,1, 5,2,0

icosashape:
' vertices
data 12
data 0,0.528,0.852, 0,-0.528,0.852, 0,0.528,-0.852, 0,-0.528,-0.852
data 0.528,0.852,0, -0.528,0.852,0, 0.528,-0.852,0, -0.528,-0.852,0
data 0.852,0,0.528, 0.852,0,-0.528, -0.852,0,0.528, -0.852,0,-0.528

' faces
data 20
data  1, 6, 8,    1, 8, 0,    1, 0,10,    1,10, 7    ' vertices for
data  1, 7, 6,    6, 7, 3,    6, 3, 9,    6, 9, 8    '  each face in
data  8, 9, 4,    8, 4, 0,    0, 4, 5,    0, 5,10    '  counter-clockwise
data 10, 5,11,   10,11, 7,    7,11, 3,    3,11, 2    '  order
data  2,11, 5,    2, 5, 4,    2, 4, 9,    2, 9, 3
